from typing import Optional
import os
import requests
from requests.api import request
from requests.models import Response

from labelci.common.utils import check_response_status, save_file
from labelci.core.client.config import DEFAULT_REQUEST_TIMEOUT
from labelci.core.client.config import (
    GET_TOKEN,
    GET_PROJECT,
    GET_PROJECT_LIST,
    GET_PROJECT_TASKS,
    GET_PROJECT_TASKS_VERSION,
    GET_LABEL_CONFIG_TEMPLATE,

    PROJECT_EXPORT,
    GET_EXPORT_FORMATS,

    DATA_VERSION_LIST,
    DATA_VERSION,
    DATA_VERSION_DETAIL,
    DATA_VERSION_COPY,

    GET_TASK_ID

)
from labelci.common.exception.labelci_sdk_exception import (
    InvalidTokenException,
    ProjectNotFound
)


class DataHubClient:

    def __init__(self, url, token) -> None:
        self.url = url
        self.token = token

    def request(
            self,
            method: str,
            relative_url: str,
            params: Optional[dict] = None,
            data: Optional[dict] = None,
            files: Optional[dict] = None,
            json: Optional[dict] = None,
            # headers: Optional[dict] = None,
            timeout: Optional[int] = DEFAULT_REQUEST_TIMEOUT,
    ):
        params = params or {}
        data = data or {}
        files = files or {}
        json = json or {}
        url = self.url + relative_url
        headers = {"Authorization": "Token " +
                                    self.token} if self.token else {}

        response = requests.request(
            method,
            url,
            params=params,
            data=data,
            json=json,
            headers=headers,
            files=files,
            timeout=timeout,
        )
        check_response_status(response)
        return response

    # def get_all_annotations(self):
    #     headers = {
    #         "Authorization": "Token " + self.token
    #     }
    #     response = requests.get(url=self.url, headers=headers)
    #     check_response_status(response=response)

    #     print(response)

    # Projcet API
    def get_project_list(self):
        """Get a list of the projects that you've created.
        """
        relative_url = GET_PROJECT_LIST
        response = self.request("GET", relative_url).json()
        projects = []
        for project in response:
            projects.append({
                "id": project["id"],
                "title": project["title"],
                "description": project["description"],
                "created_by": project["created_by"]
            })
        return projects

    def create_project(self):
        pass

    def get_label_config_templates(self):
        """Get label-config templates list
        """
        relative_url = GET_LABEL_CONFIG_TEMPLATE
        response = self.request("GET", relative_url).json()

        return response

    def get_project_tasks(self, project_id, version_id=None):
        """Retrieve a paginated list of tasks for a specific project.
        """
        relative_url = GET_PROJECT_TASKS_VERSION.format(project_id, version_id) \
            if version_id else GET_PROJECT_TASKS.format(project_id)
        print("=----=: ", relative_url)
        response = self.request("GET", relative_url).json()

        return response

    # EXPORT API
    def export_project(self, project_id, export_type="JSON", download_all_tasks=False):
        """Export annotated tasks as a file in a specific format.
        """

        relative_url = PROJECT_EXPORT.format(project_id)
        export_type = export_type.upper()
        # response = self.request("GET", relative_url, params={"exportType": export_type})._content
        response = self.request("GET", relative_url,
                                params={"exportType": export_type,
                                        "download_all_tasks": download_all_tasks}).content

        return response

    def get_export_format(self, id):
        """Retrieve the available export formats for the current project.
        """
        relative_url = GET_EXPORT_FORMATS
        response = self.request("GET", relative_url).json()

        return response

    def download_file(self, path):
        relative_url = path
        response = self.request("GET", relative_url).content

        return response

    # DataVersion API
    def get_data_version_list(self, project_id):
        """get projects data version by project id.
        """
        relative_url = DATA_VERSION_LIST.format(project_id)
        response = self.request('GET', relative_url).json()

        return response

    def check_token(self):
        """Check whether the token exists
        """
        relative_url = GET_TOKEN
        response = self.request("GET", relative_url).json()
        if not response.get("token"):
            raise InvalidTokenException

    def check_project_id(self, project_id):
        """Check whether the token exists
        """
        relative_url = GET_PROJECT.format(project_id)
        response = self.request("GET", relative_url).json()
        if response.get("status_code") == 404:
            raise ProjectNotFound

    def check_task_id(self, task_id):
        """Check whether the task exists
        """
        relative_url = GET_TASK_ID.format(task_id)
        response = self.request("GET", relative_url).json()
        if response.get("status_code") == 404:
            raise ProjectNotFound(message="Task is not found.")

        return response

    def check_data_version(self, version_id):
        """Check whether the token exists
        """
        relative_url = DATA_VERSION_DETAIL.format(version_id)
        response = self.request("GET", relative_url).json()
        if response.get("status_code") == 404:
            raise ProjectNotFound(message="Data version is not found.")




# -*- coding: utf-8 -*-
import sys
from typing import Any


class XuelangCloudException:
    
    def __init__(self, code=None, message=None, requestId=None):
        self.code = code
        self.message = message
        self.requestId = requestId

    def __str__(self):
        s = "[XuelangCloudException] code:%s message:%s requestId:%s" % (
            self.code, self.message, self.requestId)
        if sys.version_info[0] < 3 and isinstance(s, str):
            return s.encode("utf8")
        else:
            return s

    def get_code(self):
        return self.code

    def get_message(self):
        return self.message

    def get_request_id(self):
        return self.requestId


# Exceptions encountered while interection with the Hub backend
class AuthenticationException(Exception):
    def __init__(self, message="Authentication failed. Please try logging in again."):
        super().__init__(message)


class AuthorizationException(Exception):
    def __init__(
        self,
        message="You are not authorized to access this resource on Activeloop Server.",
    ):
        super().__init__(message)


class InvalidPasswordException(AuthorizationException):
    def __init__(
        self,
        message="The password you provided was invalid.",
    ):
        super().__init__(message)


class CouldNotCreateNewDatasetException(AuthorizationException):
    def __init__(
        self,
        path: str,
    ):

        extra = ""
        if path.startswith("hub://"):
            extra = "Since the path is a `hub://` dataset, if you believe you should have write permissions, try running `activeloop login`."

        message = f"Dataset at '{path}' doesn't exist, and you have no permissions to create one there. Maybe a typo? {extra}"
        super().__init__(message)


class ResourceNotFoundException(Exception):
    def __init__(
        self,
        message="The resource you are looking for was not found. Check if the name or id is correct.",
    ):
        super().__init__(message)


class BadRequestException(Exception):
    def __init__(self, message):
        message = (
            f"Invalid Request. One or more request parameters is incorrect.\n{message}"
        )
        super().__init__(message)


class OverLimitException(Exception):
    def __init__(
        self,
        message="You are over the allowed limits for this operation.",
    ):
        super().__init__(message)


class ServerException(Exception):
    def __init__(self, message="Internal Activeloop server error."):
        super().__init__(message)


class BadGatewayException(Exception):
    def __init__(self, message="Invalid response from Activeloop server."):
        super().__init__(message)


class GatewayTimeoutException(Exception):
    def __init__(self, message="Activeloop server took too long to respond."):
        super().__init__(message)


class WaitTimeoutException(Exception):
    def __init__(self, message="Timeout waiting for server state update."):
        super().__init__(message)


class LockedException(Exception):
    def __init__(self, message="The resource is currently locked."):
        super().__init__(message)


class UnexpectedStatusCodeException(Exception):
    def __init__(self, message):
        super().__init__(message)


class InvalidTokenException(Exception):
    def __init__(self, message="The authentication token is empty or error."):
        super().__init__(message)


class ProjectNotFound(Exception):
    def __init__(self, message="Project is not found."):
        super().__init__(message)


class PropertyDoesNotExistError(KeyError):
    def __init__(self, property_name: str):
        super().__init__(f"Property '{property_name}' does not exist.")


class InvalidKeyTypeError(TypeError):
    def __init__(self, item: Any):
        super().__init__(
            f"Item '{str(item)}' of type '{type(item).__name__}' is not a valid key."
        )


class ClientDoesNotExistError(KeyError):
    def __init__(self):
        super().__init__("Client does not exist.")


class ResourcesDoesNotExistError(KeyError):
    def __init__(self):
        super().__init__("Resources does not exist.")



# encoding: utf-8
# @Time: 2021/9/2 11:17 上午
# @Author: Zuo
# @File: utils.py
# @Desc:


def print_(dataset):
    """Formatted print
    """
    if isinstance(dataset, list):
        for data_info in dataset:
            print(data_info)
    else:
        print(dataset)



# -*- coding: utf-8 -*-
import os
import argparse
from labelci import Dataset
from argparse import RawTextHelpFormatter
import configparser

from labelci.bin.utils import print_


def label_cli():
    parser = argparse.ArgumentParser(description="labelci", formatter_class=RawTextHelpFormatter)
    parser.add_argument("command", help="""project    Project Information
task    Task Information， '--project' is a required parameter
data    Data version Information, '--project' is a required parameter
""")
    parser.add_argument("--data-version", help="Get data version by project id")

    parser.add_argument("--url", help="Label hub url")
    parser.add_argument("--token", help="Label hub access token")

    parser.add_argument("--id", help="""list    Get a data list that you"ve created, input "list"
id    Get data by id, input id, example 1023. 
""")

    parser.add_argument("--checkout", help="Switch to a version of the project by data version")
    parser.add_argument("--project", help="The project ID field is required to get the Task/Data Version List")
    args = parser.parse_args()

    url = None
    token = None

    if os.path.exists('config.ini'):
        conf = configparser.ConfigParser()
        conf.read("config.ini")
        url = conf.get('client', 'url')
        token = conf.get('client', 'token')
    url = args.url if args.url else url
    token = args.token if args.token else token
    if not url or not token:
        return "labelci: error: the following arguments are required: --url, --token"

    if args.command == "project":
        id_ = args.id if args.id else "list"
        if id_ == "list":
            dataset = Dataset(url, token).list()
        else:
            if args.checkout:
                dataset = Dataset(url, token).show_tasks(id_, args.checkout)
            dataset = Dataset(url, token).show_tasks(id_)
        print_(dataset)
        return

    elif args.command == "task":
        id_ = args.id if args.id else "list"
        if id_ == "list":
            if not args.project:
                return "labelci: error: the following arguments are required: --project"
            dataset = Dataset(url, token).show_tasks(args.project)
        else:
            dataset = Dataset(url, token).detail_tasks(id_)
        print_(dataset)
        return

    elif args.command == "data":
        if not args.project:
            return "labelci: error: the following arguments are required: --project"
        data = Dataset(url, token).data_version(args.project)
        print_(data)
        return

    # elif args.command == "data-version":
    #     pass
    #     # dataset




# -*- coding: utf-8 -*-
from labelci.common.exception.labelci_sdk_exception import XuelangCloudException


class DataHubCredential:

    def __init__(self, accessToken) -> None:
        if accessToken is None or accessToken.strip() == "":
            raise XuelangCloudException("InvalidDataHubCredential", "accessToken is should not be none or empty")

        self.accessToken = accessToken


class AlgorithmHubCredential:

    def __init__(self, accessToken) -> None:
        if accessToken is None or accessToken.strip() == "":
            raise XuelangCloudException("InvalidAlgorithmHubCredential", "accessToken is should not be none or empty")

        self.accessToken = accessToken



# encoding: utf-8
from typing import Optional
from labelci.core.index import Index
from labelci.core.client.client import DataHubClient
from labelci.common.exception.labelci_sdk_exception import ClientDoesNotExistError, ResourcesDoesNotExistError
import numpy as np
from PIL import Image
from io import BytesIO


class Tensor:
    def __init__(
            self,
            key: str,
            data: list,
            client: Optional[DataHubClient] = None,
    ):
        self.key = key
        self.data = data or []
        self.client = client
        self.file_cache = None

    def __getitem__(self, item):
        if self.key != "labels":
            if not self.client:
                raise ClientDoesNotExistError
            if not self.data[item]:
                raise ResourcesDoesNotExistError

            response_data = self.client.download_file(self.data[item])
            f = BytesIO(response_data)
            self.file_cache = f
            return self

        return self.data[item]

    def numpy(self):
        """file transform to numpy array format.
        """
        if not self.file_cache:
            return []
        return np.array(Image.open(self.file_cache))

    def bytes(self):
        """return BytesIO object
        """
        if not self.file_cache:
            return b""
        return self.file_cache

    def remove_file_cache(self):
        self.file_cache = None

    def update_labels(self):
        """update labels
        """

        pass



# encoding: utf-8
# @Time: 2021/8/19 9:26 上午
# @Author: Zuo
# @File: test_client.py
# @Desc:
import pytest
from labelci.core.client.client import DataHubClient


def test_client_requests(url, token):

    hub_client = DataHubClient(url, token)
    hub_client.check_token()





# encoding: utf-8
# @Time: 2021/8/16 5:31 下午
# @Author: Zuo
# @File: dataset.py
# @Desc:
from typing import Optional
from labelci.core.client.client import DataHubClient
from labelci.core.index import Index
from labelci.core.tensor import Tensor
from labelci.common.utils import project_exists, get_filename, dir_exists
from labelci.common.exception.labelci_sdk_exception import PropertyDoesNotExistError


class Dataset:
    def __init__(self,
                 url: str,
                 token: str,
                 index: Optional[Index] = None,
                 ):
        self.token = token
        self.url = url
        self.client = DataHubClient(self.url, self.token)
        self.index = index or Index()
        self.tensors: dict = {}

    @property
    def num_samples(self) -> int:
        """Returns the length of the smallest tensor.
        Ignores any applied indexing and returns the total length.
        """
        return min(map(len, self.tensors.values()), default=0)

    def __len__(self):
        """Returns the length of the smallest tensor"""
        tensor_lengths = [len(tensor.data) for tensor in self.tensors.values()]

        return min(tensor_lengths, default=0)

    def __getattr__(self, key):
        data_dic = self.tensors
        if key in data_dic:
            return data_dic[key]

        return self.__dict__[key]

    def list(self):
        datasets = self.client.get_project_list()

        return datasets

    def load(self, project_id):
        """
        Return data by project_id
        """
        self.client.check_project_id(project_id)
        data = self.client.get_project_tasks(project_id)

        self._load_meta(data)

        return self

    def export(self, project_id, export_type, export_path, all_data: bool = False):
        """
        Export annotated tasks as a file in a specific format.
        """
        self.client.check_project_id(project_id)
        data = self.client.export_project(project_id, export_type, all_data)

        filename = get_filename(project_id)
        dir_exists(export_path)
        file_path = export_path + "/" + filename + "." + export_type
        with open(file_path, "w", encoding="utf8") as f:
            f.write(data.decode(encoding="utf8"))

        return file_path

    def _load_meta(self, data):
        """Load data into tensors.
        """
        images = []
        labels = []
        for item in data:
            images.append(item["data"].get("image"))
            labels.append(item["annotations"])
        self.tensors["images"] = Tensor("images", images, self.client)
        self.tensors["labels"] = Tensor("labels", labels, self.client)

    def annotation(self, ):
        # self.client.
        pass

    def show_tasks(self, project_id, version=None):
        """Get all tasks by project_id
        """
        self.client.check_project_id(project_id)

        data = self.client.get_project_tasks(project_id, version_id=version) \
            if version else self.client.get_project_tasks(project_id)

        return data

    def detail_tasks(self, task_id):
        data = self.client.check_task_id(task_id)

        return data

    def data_version(self, project_id):
        """Get data version list by project id
        """
        self.client.check_project_id(project_id)
        data = self.client.get_data_version_list(project_id)

        return data

    def checkout_version(self, project_id, version):
        """Checkout data version.
        """

        self.client.check_data_version(version)
        data = self.client.get_project_tasks(project_id, version)

        self._load_meta(data)

        return self



import setuptools


with open("README.md", "r") as fh:
    long_description = fh.read()


setuptools.setup(
    name="labelci",
    version="0.1.2",
    author="wunsch",
    author_email="wunsch0106@gmail.com",
    description="XuelangCloud vision algorithm labelci",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        "Pillow",
        "numpy"
    ],

    entry_points={
        "console_scripts": [
            "labelci=labelci.bin.cli:label_cli",
        ]
    }
)


import sys
import os
import requests
from datetime import datetime

from labelci.common.exception.labelci_sdk_exception import (
    AuthenticationException,
    AuthorizationException,
    BadGatewayException,
    BadRequestException,
    GatewayTimeoutException,
    LockedException,
    OverLimitException,
    ResourceNotFoundException,
    ServerException,
    UnexpectedStatusCodeException,
    InvalidTokenException,
)


def check_response_status(response: requests.Response):
    """Check response status and throw corresponding exception on failure."""
    code = response.status_code
    if code >= 200 and code < 300:
        return

    try:
        message = response.json()["description"]
    except Exception:
        message = " "

    if code == 400:
        if message != " ":
            sys.exit("Error: {message}")
        raise BadRequestException(message)
    elif response.status_code == 401:
        raise AuthenticationException
    elif response.status_code == 403:
        raise AuthorizationException(message)
    elif response.status_code == 404:
        if message != " ":
            raise ResourceNotFoundException(message)
        raise ResourceNotFoundException
    elif response.status_code == 423:
        raise LockedException
    elif response.status_code == 429:
        raise OverLimitException
    elif response.status_code == 502:
        raise BadGatewayException
    elif response.status_code == 504:
        raise GatewayTimeoutException
    elif 500 <= response.status_code < 600:
        raise ServerException("Server under maintenance, try again later.")
    else:
        message = f"An error occurred. Server response: {response.status_code}"
        raise UnexpectedStatusCodeException(message)


def save_file(data, export_type, save_path):
    """save file
    """
    if not os.path.exists(save_path):
        os.mkdir(save_path)
    with open(save_path) as f:
        f.write(data)

    return "ok"


def project_exists():
    pass


def get_filename(project_id):
    t = datetime.now().isoformat(sep="-")
    return "-".join(["project_id", str(project_id), t])


def dir_exists(path):
    if not os.path.exists(path):
        os.mkdir(path)



import os

DEFAULT_REQUEST_TIMEOUT = 200

# Token
GET_TOKEN = "/api/current-user/token"

# Project API
GET_PROJECT = "/api/projects/{}"
GET_PROJECT_LIST = "/api/projects/"
CREATE_PROJECT = "/api/projects/"
VALIDATE_LABEL_CONFIG = "/api/projects/validate/"
GET_PROJECT_ID = "/api/projects/{}/"
UPDATE_PROJECT = "/api/projects/{}/"
DELETE_PROJECT = "/api/projects/{}/"
GET_PROJECT_TASKS = "/api/projects/{}/tasks/"
GET_PROJECT_TASKS_VERSION = "/api/projects/{}/tasks/{}/"
DELETE_ALL_TASKS = "/api/projects/{}/tasks/"
VALIDATE_PROJECT_LABEL_CONFIG = "/api/projects/{}/validate/"
GET_LABEL_CONFIG_TEMPLATE = "/api/templates"

# Data export API
PROJECT_EXPORT = "/api/projects/{}/export"
GET_EXPORT_FORMATS = "api/projects/{}/export/formats"

# DataVersion API
DATA_VERSION_LIST = '/api/data-version/projects/{}/'
DATA_VERSION = '/api/data-version/'
DATA_VERSION_DETAIL = '/api/data-version/{}/'
DATA_VERSION_COPY = '/api/data-version/copy/{}/'

# Task API
GET_TASK_ID = '/api/tasks/{}/'
